@@ -3,6 +3,7 @@
 
 #include <cairo/cairo.h>
 #include <glib-object.h>
+#include <gio/gio.h>
 #include <time.h>
 
 void set_abort_on_error (gboolean abort_on_error);
@@ -304,6 +305,9 @@ gboolean test_obj_torture_signature_1 (TestObj    *obj,
 /* virtual */
 int        test_obj_do_matrix (TestObj *obj, const char *somestr);
 
+void test_object_null_in (TestObj *obj);
+void test_object_null_out (TestObj **obj);
+
 /* inheritance */
 #define TEST_TYPE_SUB_OBJ           (test_sub_obj_get_type ())
 #define TEST_SUB_OBJECT(object)     (G_TYPE_CHECK_INSTANCE_CAST ((object), TEST_TYPE_SUB_OBJ, TestSubObj))
@@ -335,6 +339,7 @@ typedef int (*TestCallbackUserData) (gpointer user_data);
 
 void test_simple_callback (TestSimpleCallback callback);
 int test_callback (TestCallback callback);
+int test_multi_callback (TestCallback callback);
 int test_callback_user_data (TestCallbackUserData callback,
                              gpointer user_data);
 int test_callback_destroy_notify (TestCallbackUserData callback,
@@ -345,8 +350,13 @@ int test_callback_thaw_notifications (void);
 void test_callback_async (TestCallbackUserData callback, gpointer user_data);
 int test_callback_thaw_async (void);
 
-int test_callback_infinite (TestCallbackUserData callback,
-                           gpointer user_data);
+void test_async_ready_callback (GAsyncReadyCallback callback);
+
+void test_obj_instance_method_callback (TestObj *obj, TestCallback callback);
+void test_obj_static_method_callback (TestCallback callback);
+TestObj *test_obj_new_callback (TestCallbackUserData callback,
+                                gpointer user_data,
+                                GDestroyNotify notify);
 
 /* interface */
 #define TEST_TYPE_INTERFACE              (test_interface_get_type ())
@@ -386,6 +396,25 @@ gboolean     test_wi_802_1x_get_testbool (TestWi8021x *obj);
 void         test_wi_802_1x_set_testbool (TestWi8021x *obj, gboolean v);
 int          test_wi_802_1x_static_method (int x);
 
+/* floating gobject */
+#define TEST_TYPE_FLOATING           (test_floating_get_type ())
+#define TEST_FLOATING(object)        (G_TYPE_CHECK_INSTANCE_CAST ((object), TEST_TYPE_FLOATING, TestFloating))
+#define TEST_IS_FLOATING(object)     (G_TYPE_CHECK_INSTANCE_TYPE ((object), TEST_TYPE_FLOATING))
+#define TEST_FLOATING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TEST_TYPE_FLOATING, TestFloatingClass))
+
+typedef struct
+{
+  GInitiallyUnowned parent_instance;
+} TestFloating;
+
+typedef struct
+{
+  GInitiallyUnownedClass parent_class;
+} TestFloatingClass;
+
+GType        test_floating_get_type (void);
+TestFloating* test_floating_new (void);
+
 /* Function signature torture tests */
 void test_torture_signature_0 (int        x,
                                double     *y,
