@@ -26,7 +26,7 @@
 #include "girparser.h"
 #include "girmodule.h"
 #include "girnode.h"
-#include "gtypelib.h"
+#include "gitypelib-internal.h"
 #include "config.h"
 
 #if defined(HAVE_BACKTRACE) && defined(HAVE_BACKTRACE_SYMBOLS)
@@ -881,8 +881,8 @@ start_parameter (GMarkupParseContext *context,
   const gchar *name;
   const gchar *direction;
   const gchar *retval;
-  const gchar *dipper;
   const gchar *optional;
+  const gchar *caller_allocates;
   const gchar *allow_none;
   const gchar *transfer;
   const gchar *scope;
@@ -897,9 +897,9 @@ start_parameter (GMarkupParseContext *context,
   name = find_attribute ("name", attribute_names, attribute_values);
   direction = find_attribute ("direction", attribute_names, attribute_values);
   retval = find_attribute ("retval", attribute_names, attribute_values);
-  dipper = find_attribute ("dipper", attribute_names, attribute_values);
   optional = find_attribute ("optional", attribute_names, attribute_values);
   allow_none = find_attribute ("allow-none", attribute_names, attribute_values);
+  caller_allocates = find_attribute ("caller-allocates", attribute_names, attribute_values);
   transfer = find_attribute ("transfer-ownership", attribute_names, attribute_values);
   scope = find_attribute ("scope", attribute_names, attribute_values);
   closure = find_attribute ("closure", attribute_names, attribute_values);
@@ -919,16 +919,22 @@ start_parameter (GMarkupParseContext *context,
     {
       param->in = FALSE;
       param->out = TRUE;
+      if (caller_allocates == NULL)
+	param->caller_allocates = FALSE;
+      else
+	param->caller_allocates = strcmp (caller_allocates, "1") == 0;
     }
   else if (direction && strcmp (direction, "inout") == 0)
     {
       param->in = TRUE;
       param->out = TRUE;
+      param->caller_allocates = FALSE;
     }
   else
     {
       param->in = TRUE;
       param->out = FALSE;
+      param->caller_allocates = FALSE;
     }
 
   if (retval && strcmp (retval, "1") == 0)
@@ -936,11 +942,6 @@ start_parameter (GMarkupParseContext *context,
   else
     param->retval = FALSE;
 
-  if (dipper && strcmp (dipper, "1") == 0)
-    param->dipper = TRUE;
-  else
-    param->dipper = FALSE;
-
   if (optional && strcmp (optional, "1") == 0)
     param->optional = TRUE;
   else
@@ -1744,6 +1745,9 @@ start_type (GMarkupParseContext *context,
           else
             /* If neither zero-terminated nor length nor fixed-size is given, assume zero-terminated. */
             typenode->zero_terminated = !(typenode->has_length || typenode->has_size);
+
+          if (typenode->has_size && ctx->current_typed->type == G_IR_NODE_FIELD)
+            typenode->is_pointer = FALSE;
         } else {
           typenode->zero_terminated = FALSE;
           typenode->has_length = FALSE;
