@@ -52,6 +52,7 @@ TAG_RETURNS = 'returns'
 TAG_RETURNS_ALT = 'return value'
 TAG_ATTRIBUTES = 'attributes'
 TAG_RENAME_TO = 'rename to'
+TAG_TYPE = 'type'
 
 # Options - annotations for parameters and return values
 OPT_ALLOW_NONE = 'allow-none'
@@ -200,15 +201,23 @@ class AnnotationParser(object):
             block_name, block_options = block_header, {}
         block = DocBlock(block_name, block_options)
         comment_lines = []
+        parse_parameters = True
+        canon_name = ''
         for line in comment[pos+1:].split('\n'):
             line = line.lstrip()
             line = line[2:].strip() # Skip ' *'
             if not line:
+                if parse_parameters:
+                    parse_parameters = False
                 continue
             if line.startswith('@'):
                 line = line[1:]
             elif not ': ' in line:
-                comment_lines.append(line)
+                if parse_parameters and line:
+                    if canon_name != '' and canon_name in block.tags:
+                        block.tags[canon_name].comment += ' ' + line
+                else:
+                    comment_lines.append(line)
                 continue
             tag_name, value = self._split_tag_namevalue(line)
             canon_name = tag_name.lower()
@@ -239,7 +248,7 @@ class AnnotationParser(object):
         tag.value = value
         options, rest = self._parse_options(tag.value)
         tag.options = options
-        tag.comment = rest
+        tag.comment = rest or ''
         return tag
 
     def _parse_options(self, value):
@@ -409,6 +418,9 @@ class AnnotationApplier(object):
         self._parse_node_common(prop, block)
         if block:
             prop.doc = block.comment
+        type_tag = self._get_tag(block, TAG_TYPE)
+        if type_tag:
+            prop.type = self._resolve(type_tag.value, prop.type)
 
     def _parse_callback(self, callback):
         block = self._blocks.get(callback.ctype)
@@ -587,13 +599,15 @@ class AnnotationApplier(object):
 
     def _parse_param_ret_common(self, parent, node, tag):
         options = getattr(tag, 'options', {})
-        node.direction = self._extract_direction(node, options)
+        (node.direction, node.caller_allocates) = \
+            self._extract_direction(node, options)
         container_type = self._extract_container_type(
             parent, node, options)
         if container_type is not None:
             node.type = container_type
         if node.direction is None:
             node.direction = self._guess_direction(node)
+            node.caller_allocates = False
         node.transfer = self._extract_transfer(parent, node, options)
         param_type = options.get(OPT_TYPE)
         if param_type:
@@ -608,16 +622,32 @@ class AnnotationApplier(object):
             node.doc = tag.comment
 
     def _extract_direction(self, node, options):
+        caller_allocates = False
         if (OPT_INOUT in options or
             OPT_INOUT_ALT in options):
             direction = PARAM_DIRECTION_INOUT
         elif OPT_OUT in options:
+            subtype = options[OPT_OUT]
+            if subtype is not None:
+                subtype = subtype.one()
             direction = PARAM_DIRECTION_OUT
+            if subtype in (None, ''):
+                if (node.type.name not in BASIC_GIR_TYPES) and node.type.ctype:
+                    caller_allocates = '**' not in node.type.ctype
+                else:
+                    caller_allocates = False
+            elif subtype == 'caller-allocates':
+                caller_allocates = True
+            elif subtype == 'callee-allocates':
+                caller_allocates = False
+            else:
+                raise InvalidAnnotationError(
+                    "out direction for %s is invalid (%r)" % (node, subtype))
         elif OPT_IN in options:
             direction = PARAM_DIRECTION_IN
         else:
             direction = node.direction
-        return direction
+        return (direction, caller_allocates)
 
     def _guess_array(self, node):
         ctype = node.type.ctype
@@ -629,15 +659,20 @@ class AnnotationApplier(object):
             return True
         return False
 
+    def _is_array_type(self, node):
+        if node.type.name in ['GLib.Array', 'GLib.PtrArray',
+                              'GLib.ByteArray']:
+            return True
+        if node.type.ctype in ['GArray*', 'GPtrArray*', 'GByteArray*']:
+            return True
+        return False
+
     def _extract_container_type(self, parent, node, options):
         has_element_type = OPT_ELEMENT_TYPE in options
         has_array = OPT_ARRAY in options
 
         if not has_array:
-            has_array = \
-                node.type.name in ['GLib.Array', 'GLib.PtrArray',
-                                   'GLib.ByteArray'] or \
-                node.type.ctype in ['GArray*', 'GPtrArray*', 'GByteArray*']
+            has_array = self._is_array_type(node)
 
         # FIXME: This is a hack :-(
         if (not isinstance(node, Field) and
@@ -664,9 +699,9 @@ class AnnotationApplier(object):
         else:
             array_values = {}
 
-        is_g_array = node.type.ctype.startswith('GArray*') or \
-                     node.type.ctype.startswith('GPtrArray*') or \
-                     node.type.ctype.startswith('GByteArray*')
+        is_g_array = (node.type.ctype.startswith('GArray*') or
+                      node.type.ctype.startswith('GPtrArray*') or
+                      node.type.ctype.startswith('GByteArray*'))
 
         element_type = options.get(OPT_ELEMENT_TYPE)
         if element_type is not None:
@@ -763,6 +798,10 @@ class AnnotationApplier(object):
                 node.type.ctype,
                 self._resolve(element_type[0]),
                 self._resolve(element_type[1]))
+        elif self._is_array_type(node):
+            container_type = Array(node.type.name,
+                                   node.type.ctype,
+                                   self._resolve(element_type[0]))
         else:
             print 'FIXME: unhandled element-type container:', node
         return container_type
@@ -885,6 +924,8 @@ class AnnotationApplier(object):
         elif isinstance(node, Parameter):
             if node.direction in [PARAM_DIRECTION_INOUT,
                                   PARAM_DIRECTION_OUT]:
+                if node.caller_allocates:
+                    return PARAM_TRANSFER_NONE
                 return PARAM_TRANSFER_FULL
             # This one is a hack for compatibility; the transfer
             # for string parameters really has no defined meaning.
@@ -893,7 +934,11 @@ class AnnotationApplier(object):
             else:
                 return PARAM_TRANSFER_NONE
         elif isinstance(node, Return):
-            if (node.type.canonical in BASIC_GIR_TYPES or
+            if (isinstance(node.type, Array) and
+                    node.type.element_type is not None and
+                    node.type.element_type.name == 'utf8'):
+                return PARAM_TRANSFER_FULL
+            elif (node.type.canonical in BASIC_GIR_TYPES or
                 (node.type.canonical in [TYPE_NONE, TYPE_ANY] and
                  node.type.is_const)):
                 return PARAM_TRANSFER_NONE
