@@ -35,7 +35,10 @@ def _get_cachedir():
 
     cachedir = os.path.join(homedir, '.cache')
     if not os.path.exists(cachedir):
-        os.mkdir(cachedir, 0755)
+        try:
+            os.mkdir(cachedir, 0755)
+        except OSError:
+            return None
 
     scannerdir = os.path.join(cachedir, 'g-ir-scanner')
     if not os.path.exists(scannerdir):
@@ -69,7 +72,7 @@ class CacheStore(object):
         return (os.stat(store_filename).st_mtime >=
                 os.stat(filename).st_mtime)
 
-    def _remove_filename(filename):
+    def _remove_filename(self, filename):
         try:
             os.unlink(filename)
         except IOError, e:
