@@ -97,7 +97,7 @@ def get_libtool_command(options):
     try:
         subprocess.check_call(['libtool', '--version'],
                               stdout=open(os.devnull))
-    except subprocess.CalledProcessError, e:
+    except (subprocess.CalledProcessError, OSError), e:
         # If libtool's not installed, assume we don't need it
         return None
 
