@@ -7,6 +7,7 @@ G_DEFINE_TYPE (AnnotationObject, annotation_object, G_TYPE_OBJECT);
 enum {
   PROP_0,
   PROP_STRING_PROPERTY,
+  PROP_FUNCTION_PROPERTY
 };
 
 enum {
@@ -28,6 +29,8 @@ annotation_object_set_property (GObject         *object,
     {
     case PROP_STRING_PROPERTY:
       break;
+    case PROP_FUNCTION_PROPERTY:
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -44,6 +47,8 @@ annotation_object_get_property (GObject         *object,
     {
     case PROP_STRING_PROPERTY:
       break;
+    case PROP_FUNCTION_PROPERTY:
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -128,6 +133,17 @@ annotation_object_class_init (AnnotationObjectClass *klass)
                                                         "This property is a string",
                                                         NULL,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+  /**
+   * AnnotationObject:function-property:
+   *
+   * Type: AnnotationCallback
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_FUNCTION_PROPERTY,
+                                   g_param_spec_pointer ("function-property",
+                                                         "Function property",
+                                                         "This property is a function pointer",
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 }
 
@@ -367,7 +383,8 @@ annotation_object_compute_sum  (AnnotationObject *object,
 /**
  * annotation_object_compute_sum_n:
  * @object: a #GObject
- * @nums: (array length=n_nums zero-terminated=0): Sequence of numbers
+ * @nums: (array length=n_nums zero-terminated=0): Sequence of
+ *   numbers that are zero-terminated
  * @n_nums: Length of number array
  *
  * Test taking an array with length parameter
@@ -383,7 +400,8 @@ annotation_object_compute_sum_n(AnnotationObject *object,
 /**
  * annotation_object_compute_sum_nz:
  * @object: a #AnnotationObject
- * @nums: (array length=n_nums zero-terminated=1): Sequence of numbers
+ * @nums: (array length=n_nums zero-terminated=1): Sequence of numbers that
+ * are zero-terminated
  * @n_nums: Length of number array
  *
  * Test taking a zero-terminated array with length parameter
@@ -673,4 +691,13 @@ annotation_set_source_file (const char *fname)
 {
 }
 
+/**
+ * annotation_ptr_array:
+ * @array: (element-type GLib.Value): the array
+ */
+void
+annotation_ptr_array (GPtrArray *array)
+{
+}
+
 char backslash_parsing_tester_2 = '\\';
